package Compiladores;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.util.Hashtable;

import javax.swing.JOptionPane;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * Clase para validar el documento XML y obtener los datos de la cabecera de configuracin
 * @author  Enrique Marquet Durn
 * @version 1.0
 */
public class Analyser {
	Document documentoValidado;
	private String idioma;  // Idioma de la voz a utilizar en la conversin, determinado por la configuracin
	private Boolean imagen;
	private Boolean unico;
	private String prefijo;
	private Boolean nota;
	private Hashtable<String, String> tablaIdioma; // Se guarda los cdigos de los idiomas y su descripcin
	
	/**
	 * Constructor de la clase
	 * @param pathXMLIn el <code>String</code> con el path donde se encuentra el documento XML a validar
	 */
	public Analyser (String pathXMLIn) {
		tablaIdioma = new Hashtable<String, String>();
		analyzeXML(pathXMLIn);
		contruirTabla();
	}
	
	/**
	 * Getter para el documento validado
	 * @return Document el documento validado
	 */
	public Document getDocumentoValidado() {
		return documentoValidado;
	}

	/**
	 * Informa las variables globales para la configuracin del la ejecucin del proceso
	 * @return resultado es un <code>int</code>, si la configuracin es correcta devuelve 0, en otros casos devuelve el valor del error
	 * de esta manera la clase que lo llama podr saber qu error se ha producido
	 */
	public int informarConfig () {
		int resultado = 0;
		XPathFactory aXPF = XPathFactory.newInstance();
        XPath aXPath = aXPF.newXPath();
        try {
			XPathExpression aExpr = aXPath.compile("/Audio/Idioma");
			Node aNodo = (Node) aExpr.evaluate(documentoValidado,XPathConstants.NODE);
			String valor = aNodo.getTextContent();
            if (!this.tablaIdioma.containsKey(valor)) {
            	resultado = 1;
            	return resultado;
            }
            setIdioma(valor);
            
			aExpr = aXPath.compile("/Audio/Imagen");
			aNodo = (Node) aExpr.evaluate(documentoValidado,XPathConstants.NODE);
			valor = new String (aNodo.getTextContent());
			if (valor.trim().equals("SI")) {
				setImagen(true);
			} else {
				if (valor.trim().equals("NO")) {
				setImagen(false);
				}
				else {
					resultado = 2;
					return resultado;
				}
			}
				
			aExpr = aXPath.compile("/Audio/Unico");
			aNodo = (Node) aExpr.evaluate(documentoValidado,XPathConstants.NODE);
			valor = aNodo.getTextContent();
			if (valor.trim().equals("SI")) {
				setUnico(true);
			} else {
				if (valor.trim().equals("NO")) {
					setUnico(false);
				} else {
					resultado = 3;
					return resultado;
				}
			}
			
			aExpr = aXPath.compile("/Audio/Prefijo");
			aNodo = (Node) aExpr.evaluate(documentoValidado,XPathConstants.NODE);
			valor = aNodo.getTextContent();
			if (valor.trim().equals("")) {
				resultado = 4;
				return resultado;
			}
			setPrefijo(valor);
			
			aExpr = aXPath.compile("/Audio/Nota");
			aNodo = (Node) aExpr.evaluate(documentoValidado,XPathConstants.NODE);
			valor = aNodo.getTextContent();
			if (valor.trim().equals("SI")) {
				setNota(true);
			} else {
				if (valor.trim().equals("NO")) {
					setNota(false);
				} else {
					resultado = 5;
					return resultado;
				}
			}
			
		} catch (XPathExpressionException e) {
			JOptionPane.showMessageDialog(null, e);
			resultado = 6;
			return resultado;
		}
		return resultado;
	}
	/**
	 * Validacin del documento de configuracin XML con la DTD
	 * @param fXMLIn Ubicacin y nombre del archivo XML de configuracin a validar
	 */
    private void analyzeXML(String fXMLIn)  {
    	BufferedReader reader = null;

        try
        {
            // Get xml parser (with DTD validation activated)
            DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
            dbFactory.setValidating(true);
            dbFactory.setNamespaceAware(true);
            DocumentBuilder db = dbFactory.newDocumentBuilder();
            ErrorHandler eh = new MyErrorHandler();
            db.setErrorHandler(eh);

            StringBuilder sb = new StringBuilder();
            boolean doctype_added = false;
            reader = new BufferedReader(new InputStreamReader(new FileInputStream(fXMLIn), "UTF8"));
            for(String line = reader.readLine(); line != null; line = reader.readLine())
            {
                // Adds doctype declaration after xml declaration (if present)
                if (! (doctype_added || line.startsWith("<?xml version")))
                {
                    doctype_added = true;
                    sb.append("<!DOCTYPE Audio SYSTEM \"Audio.dtd\">\n");
                }
                sb.append(line);
                sb.append("\n");
            }

            // System.out.println("\n Contenido de sb = "+ sb.toString());
            // Parse xml input file obtaining xml doc
            documentoValidado = db.parse(new InputSource(new StringReader(sb.toString())));

        }
        catch (java.lang.Exception e)
        {
        	JOptionPane.showMessageDialog(null, "Error validando XML.", "Error validacin", JOptionPane.ERROR_MESSAGE);
            System.exit(1);
        } finally {
            try {
                reader.close();
            } catch (IOException e) {
            	JOptionPane.showMessageDialog(null, "Error cerrando BufferedReader.", "Error", JOptionPane.ERROR_MESSAGE);
            }
        }

    }


    /**
     * 
     * Clase que gestiona los errores de la validacin del fichero XML
     *
     */
    // Handles DTD validation errors
    private static class MyErrorHandler implements ErrorHandler
    {
        public void warning(SAXParseException e) throws SAXException
        {
            printErrorInfo(e, "Warning processing XML:");
        }
        public void error(SAXParseException e) throws SAXException
        {
            printErrorInfo(e, "Error processing XML:");
            System.exit(1);
        }
        public void fatalError(SAXParseException e) throws SAXException
        {
            printErrorInfo(e, "Fatal error processing XML:");
            System.exit(1);
        }
        private void printErrorInfo(SAXParseException e, String tipoError)
        {
            String messageError = new String("   Line number: "+e.getLineNumber() + "\n   Column number: "+e.getColumnNumber() +
            		"\n   Message: "+e.getMessage());
        	JOptionPane.showMessageDialog(null, messageError, tipoError, JOptionPane.ERROR_MESSAGE);
        }
    }
	
	/*
	 * Construye una tabla con todos los idiomas que acepta el sistema
	 * la tabla se utilizar para validar el documento de configuracin
	 * los valores son los que acepta Google
	 */
    private void contruirTabla() {
    	this.tablaIdioma.put("es", "Castellano");
    	this.tablaIdioma.put("af", "Afrikaans");
    	this.tablaIdioma.put("sq", "Albanian");
    	this.tablaIdioma.put("ar", "Arabic");
    	this.tablaIdioma.put("az", "Azerbaijani");
    	this.tablaIdioma.put("eu", "Basque");
    	this.tablaIdioma.put("bn", "Bengali");
    	this.tablaIdioma.put("be", "Belarusian");
    	this.tablaIdioma.put("bg", "Bulgarian");
    	this.tablaIdioma.put("ca", "Catalan");
    	this.tablaIdioma.put("zh-CN", "Chinese Simplified");
    	this.tablaIdioma.put("zh-TW", "Chinese Traditional");
    	this.tablaIdioma.put("hr", "Croatian");
    	this.tablaIdioma.put("cs", "Czech");
    	this.tablaIdioma.put("da", "Danish");
    	this.tablaIdioma.put("nl", "Aleman");
    	this.tablaIdioma.put("en", "Ingls");
    	this.tablaIdioma.put("eo", "Esperanto");
    	this.tablaIdioma.put("et", "Estonian");
    	this.tablaIdioma.put("tl", "Filipino");
    	this.tablaIdioma.put("fi", "Finnish");
    	this.tablaIdioma.put("fr", "Francs");
    	this.tablaIdioma.put("gl", "Gallego");
    	this.tablaIdioma.put("ka", "Georgian");
    	this.tablaIdioma.put("de", "German");
    	this.tablaIdioma.put("el", "Greek");
    	this.tablaIdioma.put("gu", "Gujarati");
    	this.tablaIdioma.put("ht", "Haitian Creole");
    	this.tablaIdioma.put("iw", "Hebrew");
    	this.tablaIdioma.put("hi", "Hindi");
    	this.tablaIdioma.put("hu", "Hungarian");
    	this.tablaIdioma.put("is", "Icelandic");
    	this.tablaIdioma.put("id", "Indonesian");
    	this.tablaIdioma.put("ga", "Irish");
    	this.tablaIdioma.put("it", "Italiano");
    	this.tablaIdioma.put("ja", "Japanese");
    	this.tablaIdioma.put("kn", "Kannada");
    	this.tablaIdioma.put("ko", "Korean");
    	this.tablaIdioma.put("la", "Latin");
    	this.tablaIdioma.put("lv", "Latvian");
    	this.tablaIdioma.put("lt", "Lithuanian");
    	this.tablaIdioma.put("mk", "Macedonian");
    	this.tablaIdioma.put("ms", "Malay");
    	this.tablaIdioma.put("mt", "Maltese");
    	this.tablaIdioma.put("no", "Norwegian");
    	this.tablaIdioma.put("fa", "Persian");
    	this.tablaIdioma.put("pl", "Polish");
    	this.tablaIdioma.put("pt", "Portugus");
    	this.tablaIdioma.put("ro", "Romanian");
    	this.tablaIdioma.put("ru", "Russian");
    	this.tablaIdioma.put("sr", "Serbian");
    	this.tablaIdioma.put("sk", "Slovak");
    	this.tablaIdioma.put("sl", "Slovenian");
    	this.tablaIdioma.put("sw", "Swahili");
    	this.tablaIdioma.put("sv", "Swedish");
    	this.tablaIdioma.put("ta", "Tamil");
    	this.tablaIdioma.put("te", "Telugu");
    	this.tablaIdioma.put("th", "Thai");
    	this.tablaIdioma.put("tr", "Turkish");
    	this.tablaIdioma.put("uk", "Ukrainian");
    	this.tablaIdioma.put("ur", "Urdu");
    	this.tablaIdioma.put("vi", "Vietnamese");
    	this.tablaIdioma.put("cy", "Welsh");
    	this.tablaIdioma.put("yi", "Yiddish");
    }

	/**
	 * Getter para el idioma seleccionado
	 * @return the idioma
	 */
	public String getIdioma() {
		return idioma;
	}

	/**
	 * Setter para la clave del idioma.
	 * @param idioma the idioma to set
	 */
	public void setIdioma(String idioma) {
		this.idioma = idioma;
	}


	/**
	 * Devuelve la descripcin del texto del idioma de configuracin, a partir de los valores de la tabla
	 * @return valor texto del idioma
	 */
	public String getIdiomaText() {
		String resultado = new String ();
		if (this.tablaIdioma.containsKey(idioma)) {
			resultado = this.tablaIdioma.get(idioma);
        }
		return resultado;
	}

	/**
	 * Getter para el campo Imagen del fichero de configuracin
	 * @return the imagen
	 */
	public Boolean getImagen() {
		return imagen;
	}

	/**
	 * Setter para el campo Imagen de configuracin
	 * @param imagen the imagen to set
	 */
	public void setImagen(Boolean imagen) {
		this.imagen = imagen;
	}

	/**
	 * Getter para el campo Unico del fichero de configuracin
	 * @return the unico
	 */
	public Boolean getUnico() {
		return unico;
	}

	/**
	 * Setter para el campo Unico de configuracin
	 * @param unico the unico to set
	 */
	public void setUnico(Boolean unico) {
		this.unico = unico;
	}

	/**
	 * Getter para el campo Prefijo del fichero de configuracin
	 * @return the prefijo
	 */
	public String getPrefijo() {
		return prefijo;
	}

	/**
	 * Setter para el campo Prefijo de configuracin
	 * @param prefijo the prefijo to set
	 */
	public void setPrefijo(String prefijo) {
		this.prefijo = prefijo;
	}

	/**
	 * Getter para el campo Nota del fichero de configuracin
	 * @return the nota
	 */
	public Boolean getNota() {
		return nota;
	}

	/**
	 * Setter para el campo Nota de configuracin
	 * @param nota the nota to set
	 */
	public void setNota(Boolean nota) {
		this.nota = nota;
	}
}
