package Compiladores;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

import javax.swing.JOptionPane;

import org.apache.commons.io.FileUtils;

/**
 * Clase para gestionar los ficheros de la aplicacin
 * @author  Enrique Marquet Durn
 * @version 1.0
 */
public class GestFiles {

	// Campos utilizados por la clase, slo accesibles dentro de la misma clase
	private String nombreFicheroEntrada;
	private String nombreDirSalida;
	private Path inputPath;
	private Path outputPath;
	private Path dirTmp;

	/**
	 * Constructor
	 * 
	 */
	public GestFiles(){
		this.nombreFicheroEntrada = new String ();
		this.nombreDirSalida = new String ();

	}

	/**
	 * Inicializa las rutas, ficheros (entrada y temporal)
	 * @param nombreFicheroEntrada es el <code>String</code> que contiene el nombre del fichero de entrada
	 * @param nombreDirSalida es el <code>String</code> que contiene el nombre del directorio de salida
	 */
	public void initialize (String nombreFicheroEntrada, String nombreDirSalida){
		this.nombreFicheroEntrada = nombreFicheroEntrada;
		this.nombreDirSalida = nombreDirSalida;
		inputPath = Paths.get(this.nombreFicheroEntrada);
		outputPath = Paths.get(this.nombreDirSalida);
	}
	
	/**
	 * Comprueba que la ruta del fichero de salida exista, si no existe la crea
	 */
	public void generarDirs() {
		File directorioSalida = new File(outputPath.toString());
		if (!directorioSalida.exists()) {
			directorioSalida.mkdirs();
		}
		try {
			dirTmp = Files.createTempDirectory("audiosTmp");
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, e, "Error Proceso", JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
	}
	
	/**
	 * Borra el directorio de trabajo con los ficheros que hayan dentro
	 */
	public void delete() {
			FileUtils.deleteQuietly(dirTmp.toFile());
	}
	
	/**
	 * Getter para el nombre del fichero de entrada y su ruta
	 * @return the nombreFicheroEntrada
	 */
	public String getNombreFicheroEntrada() {
		return nombreFicheroEntrada;
	}

	/**
	 * Setter para el nombre del fichero de entrada y su ruta
	 * @param nombreFicheroEntrada the nombreFicheroEntrada to set
	 */
	public void setNombreFicheroEntrada(String nombreFicheroEntrada) {
		this.nombreFicheroEntrada = nombreFicheroEntrada;
		setInputPath(Paths.get(this.nombreFicheroEntrada));
	}

	/**
	 * Getter para el nombre del directorio de salida
	 * @return the nombreDirSalida
	 */
	public String getNombreDirSalida() {
		return nombreDirSalida;
	}

	/**
	 * Setter para el nombre del directorio de salida
	 * @param nombreDirSalida the nombreDirSalida to set
	 */
	public void setNombreDirSalida(String nombreDirSalida) {
		this.nombreDirSalida = nombreDirSalida;
		setOutputPath(Paths.get(nombreDirSalida));
	}

	/**
	 * Getter para el <Path> para el directorio de salida
	 * @return the outputPath
	 */
	public Path getOutputPath() {
		return outputPath;
	}

	/**
	 * Setter para el <Path> para el directorio de salida
	 * @param outputPath the outputPath to set
	 */
	public void setOutputPath(Path outputPath) {
		this.outputPath = outputPath;
	}
	/**
	 * Getter para el <Path> para la ubicacin y nombre del documento de texto de entrada
	 * @return the inputPath
	 */
	public Path getInputPath() {
		return inputPath;
	}
	/**
	 * Setter para el <Path> para la ubicacin y nombre del documento de texto de entrada
	 * @param inputPath the inputPath to set
	 */
	public void setInputPath(Path inputPath) {
		this.inputPath = inputPath;
	}
	/**
	 * Getter para el <Path> para la ubicacin del directorio temporal
	 * @return the dirTmp
	 */
	public Path getDirTmp() {
		return dirTmp;
	}
	/**
	 * Setter para el <Path> para la ubicacin del directorio temporal
	 * @param dirTmp the dirTmp to set
	 */
	public void setDirTmp(Path dirTmp) {
		this.dirTmp = dirTmp;
	}


}
