package Compiladores;

import java.nio.file.Path;
import java.nio.file.Paths;

import javax.swing.JOptionPane;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

import org.odftoolkit.odfdom.dom.element.dr3d.Dr3dSceneElement;
import org.odftoolkit.odfdom.dom.element.draw.*;
import org.odftoolkit.odfdom.dom.element.office.OfficeAnnotationElement;
import org.odftoolkit.odfdom.dom.element.office.OfficeTextElement;
import org.odftoolkit.odfdom.dom.element.table.*;
import org.odftoolkit.odfdom.dom.element.text.*;
import org.odftoolkit.odfdom.pkg.OdfElement;
import org.odftoolkit.simple.TextDocument;
import org.odftoolkit.simple.text.TextHyperlink;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Clase que realiza el tratamiento del documento de texto ODF
 * @author  Enrique Marquet Durn
 * @version 1.0
 */
public class OpenDocument {

	TextDocument outputDocument; // Documento OpenDocument a tratar
	String dirDestino;          // Directorio a grabar los archivos de audio
	String nombreArch;          // Nombre del archivo de audio a grabar
	private Analyser analiser; // Contiene toda la informacin sobre el documento de configuracin
	int contGeneral = 0;     // Contador general de separacin de archivos
	int contParrafo = 0;     // Contador para separacin de prrafos en archivos
	int contHeadings = 0;    // Contador para separacin de headings en archivos
	int contListas = 0;      // Contador para separacin de listas en archivos
	int contTablas = 0;      // Contador para separacin de tablas en archivos
	int contIndices = 0;     // Contador para separacin de indices en archivos
	int contSeccion = 0;     // Contador para separacin de secciones en archivos
	int contTodo = 0;        // Contador para separacin de todo en archivos
	int contGrupo = 0;       // Contador para los grupos realizados
	Boolean primeraTrad = false;  // Para saber si se ha llamado alguna vez al proceso TTS
	ProcesoTTS procesoTTS;    // Se guarda la clase del ProcesoTTS que se utiliza para la conversin
	OfficeTextElement contentroot = null;  // Contenido del archivo content.xml de la etiqueta office:text
	

	/**
	 * Constructor de la clase
	 * @param pathDocTrab es el <code>String</code> que contiene el nombre del archivo de texto a tratar
	 */
	public OpenDocument (String pathDocTrab) {
		try {
			this.outputDocument =  TextDocument.loadDocument(pathDocTrab);
			// Lo primero que hay que hacer es obtener el contenido del archivo content.xml
			// coge el contenido de la etiqueta office:text
			contentroot = outputDocument.getContentRoot();
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, e);
			return;
		}

	}
	
	/**
	 * Proceso de tratamiento del documento de configuracin, donde 
	 * se seleccionarn los trozos de texto que se tienen que traducir llamando al procedimiento procDocumento
	 * @param analise es el <code>Analyser</code> que contiene la configuracin
	 * @param dirDest es el <code>String</code> con el directorio destino para guardar los archivos
	 * @param dirTmp es el <code>Path</code> con el directorio temporal para guardar los archivos intermedios de audio
	 * 
	 */
	public void trtDocumento (Analyser analise, String dirDest, Path dirTmp) {
		this.analiser = analise;
		this.dirDestino = dirDest;
		String tipo = null;
		String auxStr = null;
		int rangoIni = 0;
		int rangoFin = Integer.MAX_VALUE;
		String prefRango = null;
		try {
			
			// Empieza el tratamiento del documento xml de configuracin 
			// Se obtienen los nodos Tgrupo para su tratamiento para TTS
			XPathFactory aXPF = XPathFactory.newInstance();
	        XPath aXPath = aXPF.newXPath();
	        XPathExpression aExpr = aXPath.compile("/Audio/Traducir/Tgrupo");
	        NodeList aNodes = (NodeList) aExpr.evaluate(analiser.getDocumentoValidado(), XPathConstants.NODESET);
	        // Tenemos la lista de nodos Tgrupo que habr que traducir en el proceso TTS
	        for (int i=0; i<aNodes.getLength(); i++)
	        {
        		contGeneral++;
	        	Node aNode = aNodes.item(i);
	        	
	        	// Obtenemos los nodos de un Tgrupo
	        	NodeList aChildren1 = aNode.getChildNodes();
	        	
	        	prefRango = "";
	        	
	        	// Creamos una instancia para la traduccin TTS
	        	nombreArch = new String (analiser.getPrefijo().concat("G").concat(String.valueOf(contGeneral)));
	        	String nombreAudio = Paths.get(dirDestino, nombreArch).toAbsolutePath().toString() ;
				procesoTTS = new ProcesoTTS(nombreAudio, dirTmp);

	        	for(int k = 0; k < aChildren1.getLength(); k++) {
	        		
		        	if (aChildren1.item(k).getNodeName() == "Tipo") {
		        		tipo = aChildren1.item(k).getTextContent();
		        	}

		        	if (aChildren1.item(k).getNodeName() == "Rango") {
	        			Node infoNode = aChildren1.item(k);
	        			// Obtencin de la informacin del rango a utilizar
	        			NodeList aChildren2 = infoNode.getChildNodes();
	        			for(int j = 0; j < aChildren2.getLength(); j++) {
	        				// Obtenemos el rango inicial
	        				if (aChildren2.item(j).getNodeName() == "Inicio") {
	        					auxStr = aChildren2.item(j).getTextContent();
	        					// Hay que comprobar que el rango inicial es un numrico entero
	        					if (auxStr.trim().matches ("\\d+$")) {
	        						rangoIni = Integer.parseInt(auxStr.trim());
	        					}
	        					else {
	        						rangoIni = 1;
	        					}
	        				}
	        				// obtenemos el final del rango
	        				if (aChildren2.item(j).getNodeName() == "Final") {
	        					auxStr = aChildren2.item(j).getTextContent();
	        					// Hay que comprobar que el rango inicial es un numrico entero
	        					if (auxStr.trim().matches ("\\d+$")) {
	        						rangoFin = Integer.parseInt(auxStr.trim());
	        					}
	        					else {
	        						// Si no es numrico se considera que el final es el final del documento
	        						// con lo que asignamos el valor 0
	        						rangoFin = 0;
	        					}
	        				}
	        				// Ahora se obtiene el prefijo del rango si lo hay
	        				if (aChildren2.item(j).getNodeName() == "Prefijo-rango") {
	        					prefRango = aChildren2.item(j).getTextContent();
	        				}
	        				
	        			}
	        			// Ya tenemos un rango a tratar, hay que proceder a buscar este grupo en el documento
	        			// si se encuentra algo habr que realizar la conversin TTS
	        			contGrupo++;
	        			auxStr = "";
	        			if (tipo.equals("pa")) {
	        				contParrafo++;
	        				auxStr = new String("PA".concat(String.valueOf(contParrafo)));
	        			}
	        			if (tipo.equals("he")) {
	        				contHeadings++;
	        				auxStr = new String("HE".concat(String.valueOf(contHeadings)));
	        			}
	        			if (tipo.equals("li")) {
	        				contListas++;
	        				auxStr = new String("LI".concat(String.valueOf(contListas)));
	        			}
	        			if (tipo.equals("ta")) {
	        				contTablas++;
	        				auxStr = new String("TA".concat(String.valueOf(contTablas)));
	        			}
	        			if (tipo.equals("tc")) {
	        				contIndices++;
	        				auxStr = new String("TC".concat(String.valueOf(contIndices)));
	        			}
	        			if (tipo.equals("se")) {
	        				contSeccion++;
	        				auxStr = new String("SE".concat(String.valueOf(contSeccion)));
	        			}
	        			if (tipo.equals("to")) {
	        				contTodo++;
	        				auxStr = new String("TO".concat(String.valueOf(contTodo)));
	        			}
	        			
	        			procDocumento(tipo, rangoIni, rangoFin, prefRango, auxStr);
	        			if (!analiser.getUnico()) {
	        				procesoTTS.generaAif();
	        			}
	        		}
	        	}
	        	if (analiser.getUnico()) {
    				procesoTTS.generaAif();
	        	}

	        }
	        	
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, e);
			return;
		}
		
	}

	/**
	 * Cierre del documento odt
	 */
	public void close() {
		outputDocument.close();
	}
	
	/**
	 * Proceso de bsqueda de los formatos de texto a convertir en voz
	 * que se tienen que encontrar dentro del rango recibido
	 * @param tipo es el <code>String</code> que contiene el tipo a buscar en el documento
	 * @param rangoIni el <code>int</code> valor del rango inicial a empezar a seleccionar texto
	 * @param rangoFin el <code>int</code> valor del rango final del texto seleccionado
	 * @param prefRango el <code>String</code> prefijo a considerar para la parte del nombre que pertenece al rango del audio 
	 */
	private void procDocumento(String tipo, int rangoIni, int rangoFin, String prefRango, String auxStr) {
		OdfElement elementoTrt = null;
		int contador = 0; // Contador de los elementos que debe tratar
		int contLeidos;   // Contador de elementos leidos de la estructura del documento
		
		// Construccin del nombre del archivo que contendr este rango
		// slo en el caso de que se haya elegido mltiples archivos
		// En el caso de que no se haya elegido mltiples archivos el nombre ser el general del grupo
		if (!analiser.getUnico()) {
			if (prefRango.length() > 0)
				nombreArch = new String (analiser.getPrefijo().concat("G").concat(String.valueOf(contGeneral)).concat(prefRango) );
			else {
				
				nombreArch = new String (analiser.getPrefijo().concat("G").concat(String.valueOf(contGeneral)).concat(auxStr));
			}
        	String nombreAudio = Paths.get(dirDestino, nombreArch).toAbsolutePath().toString() ;

			procesoTTS.setNomArch(nombreAudio);
			procesoTTS.setNuevoArch(true);
		}
		
		// Proceso para tratar elemento a elemento del contenido del documento
		elementoTrt = (OdfElement) contentroot.getFirstElementChild();
		int numElem = contentroot.getChildElementCount();
		contLeidos = 1;
		
		while (contLeidos<= numElem && (contador < rangoFin || rangoFin == 0)) {
			if ((tipo.equals("pa") || tipo.equals("to")) && elementoTrt instanceof TextPElement) {
				contador++;
				if (contador >= rangoIni) {
					procElementoP(elementoTrt);
				}
			}
			if ((tipo.equals("he") || tipo.equals("to")) && elementoTrt instanceof TextHElement) {
				contador++;
				if (contador >= rangoIni) {
					procElementoP(elementoTrt); // Nos sirve el mismo tratamiento que para un parrafo ya que la diferencia es mnima
				}
			}
			if ((tipo.equals("li") || tipo.equals("to")) && elementoTrt instanceof TextListElement) {
				contador++;
				if (contador >= rangoIni) {
					procElementoLista(elementoTrt);
				}
			}
			if ((tipo.equals("ta") || tipo.equals("to")) && elementoTrt instanceof TableTableElement) {
				contador++;
				if (contador >= rangoIni) {
					procElementoTabla(elementoTrt);
				}
			}
			if ((tipo.equals("tc") || tipo.equals("to")) && elementoTrt instanceof TextTableOfContentElement) {
				contador++;
				if (contador >= rangoIni) {
					procElementoIndice(elementoTrt);
				}
			}
			if ((tipo.equals("se") || tipo.equals("to")) && elementoTrt instanceof TextSectionElement) {
				contador++;
				if (contador >= rangoIni) {
					procElementoSeccion(elementoTrt);
				}
			}
			contLeidos++;
			elementoTrt = (OdfElement) elementoTrt.getNextElementSibling();
		}
	}
	
	/**
	 * Proceso para buscar el texto a convertir en voz, para un elemento
	 * prrafo  o heading
	 * @param odfElement es el <code>OdfElement</code> que contiene el nodo a tratar
	 */
	private void procElementoP(OdfElement odfElement) {
		OdfElement elementoTrt = null;
		String textoTTS = null;
		int numElem;
		int numNodos;
		
		elementoTrt = odfElement;
		numElem = elementoTrt.getChildElementCount();
		numNodos = elementoTrt.getLength();
		// Si se trata de un elemento vacio, no tenemos texto
		if (numNodos == 0) {
			return;
		}
		
		// Si numElem es cero quiere decir que no tiene elementos hijo, pero si contenido de texto
		if (numElem == 0) {
			textoTTS = elementoTrt.getTextContent();
			procesoTTS.textToSpeech(textoTTS, analiser.getIdioma());
			return;
		} 
		
		// Si llegamos aqu quiere decir que el elemento prrafo tiene hijos y puede que l mismo tenga texto
		// habr que tratar cada nodo hijo para ver si se tiene que tener en cuenta los respectivos textos encontrados
		textoTTS = obtenerTexto(elementoTrt);

		procesoTTS.textToSpeech(textoTTS, analiser.getIdioma());
	}

	/**
	 * Proceso para buscar el texto a convertir en voz, para un elemento
	 * tipo Lista
	 * @param odfElement es el <code>OdfElement</code> que contiene el nodo a tratar
	 */
	private void procElementoLista(OdfElement odfElement) {
		OdfElement elementoTrt = null;
		String textoTTS = null;
		int numElem;
		int numNodos;
		
		elementoTrt = odfElement;
		numElem = elementoTrt.getChildElementCount();
		numNodos = elementoTrt.getLength();
		// Si se trata de un elemento vacio, no tenemos texto
		if (numNodos == 0) {
			return;
		}
		
		// Si numElem es cero quiere decir que no tiene elementos hijo, pero si contenido de texto
		if (numElem == 0) {
			textoTTS = elementoTrt.getTextContent();
			procesoTTS.textToSpeech(textoTTS, analiser.getIdioma());
			return;
		} 
		// Si llegamos aqu quiere decir que el elemento tiene elementos hijo y puede que l mismo tenga texto
		// habr que tratar cada nodo hijo para ver si se tiene que tener en cuenta los respectivos textos encontrados
		StringBuilder sb = new StringBuilder();
		
		NodeList nodeList = elementoTrt.getChildNodes();
		int i;
		for (i = 0; i < nodeList.getLength(); i++) {
			Node node;
			node = nodeList.item(i);
			if (node.getNodeType() == Node.TEXT_NODE)
				sb.append(node.getNodeValue());
			else if (node.getNodeType() == Node.ELEMENT_NODE) {
				if (node instanceof TextListHeaderElement) { 
					// Se deja un espacio para separar textos de las listas, sino pronuncia todo junto
					sb.append(' ');
				}
				else if (node instanceof TextListItemElement) {
					// Se deja un espacio para separar textos de las listas, sino pronuncia todo junto
					sb.append('\n');
				}
				if (((OdfElement)node).getLength() > 0) { 
					// Se evita tratar elementos vacios
					sb.append(obtenerTexto((OdfElement)node));
				}

			}
		}
		textoTTS = sb.toString();
		
		procesoTTS.textToSpeech(textoTTS, analiser.getIdioma());
	}

	/**
	 * Proceso para buscar el texto a convertir en voz, para un elemento
	 * tipo Tabla
	 * @param odfElement es el <code>OdfElement</code> que contiene el nodo a tratar
	 */
	private void procElementoTabla(OdfElement odfElement) {
		OdfElement elementoTrt = null;
		String textoTTS = null;
		int numElem;
		int numNodos;
		
		elementoTrt = odfElement;
		numElem = elementoTrt.getChildElementCount();
		numNodos = elementoTrt.getLength();
		// Si se trata de un elemento vacio, no tenemos texto
		if (numNodos == 0) {
			return;
		}
		
		// Si numElem es cero quiere decir que no tiene elementos hijo, pero si contenido de texto
		if (numElem == 0) {
			textoTTS = elementoTrt.getTextContent();
			procesoTTS.textToSpeech(textoTTS, analiser.getIdioma());
			return;
		} 
		// Si llegamos aqu quiere decir que el elemento tiene elementos hijo y puede que l mismo tenga texto
		// habr que tratar cada nodo hijo para ver si se tiene que tener en cuenta los respectivos textos encontrados
		textoTTS = obtenerTextoTabla(elementoTrt);
		
		procesoTTS.textToSpeech(textoTTS, analiser.getIdioma());
	}
	
	/**
	 * Proceso para buscar el texto a convertir en voz, para un elemento
	 * tipo Indice (tabla de contenido)
	 * @param odfElement es el <code>OdfElement</code> que contiene el nodo a tratar
	 */
	private void procElementoIndice(OdfElement odfElement) {
		OdfElement elementoTrt = null;
		String textoTTS = null;
		int numElem;
		int numNodos;
		
		elementoTrt = odfElement;
		numElem = elementoTrt.getChildElementCount();
		numNodos = elementoTrt.getLength();
		// Si se trata de un elemento vacio, no tenemos texto
		if (numNodos == 0) {
			return;
		}
		
		// Si numElem es cero quiere decir que no tiene elementos hijo, pero si contenido de texto
		if (numElem == 0) {
			textoTTS = elementoTrt.getTextContent();
			procesoTTS.textToSpeech(textoTTS, analiser.getIdioma());
			return;
		} 
		// Si llegamos aqu quiere decir que el elemento tiene elementos hijo y puede que l mismo tenga texto
		// habr que tratar cada nodo hijo para ver si se tiene que tener en cuenta los respectivos textos encontrados
		textoTTS = obtenerTextoIndice(elementoTrt);
		
		procesoTTS.textToSpeech(textoTTS, analiser.getIdioma());
	}

	/**
	 * Proceso para buscar el texto a convertir en voz, para un elemento
	 * tipo Section
	 * @param odfElement es el <code>OdfElement</code> que contiene el nodo a tratar
	 */
	private void procElementoSeccion(OdfElement odfElement) {
		OdfElement elementoTrt = null;
		String textoTTS = null;
		int numElem;
		int numNodos;
		
		elementoTrt = odfElement;
		numElem = elementoTrt.getChildElementCount();
		numNodos = elementoTrt.getLength();
		// Si se trata de un elemento vacio, no tenemos texto
		if (numNodos == 0) {
			return;
		}
		
		// Si numElem es cero quiere decir que no tiene elementos hijo, pero si contenido de texto
		if (numElem == 0) {
			textoTTS = elementoTrt.getTextContent();
			procesoTTS.textToSpeech(textoTTS, analiser.getIdioma());
			return;
		} 
		// Si llegamos aqu quiere decir que el elemento tiene elementos hijo y puede que l mismo tenga texto
		// habr que tratar cada nodo hijo para ver si se tiene que tener en cuenta los respectivos textos encontrados
		textoTTS = obtenerTextoSeccion(elementoTrt);
		
		procesoTTS.textToSpeech(textoTTS, analiser.getIdioma());
	}
	
	
	/**
	 * Proceso para buscar el texto a convertir en voz, para un elemento de dibujo,
	 * Se separa el inicio del proceso de lo que es el prrafo porque est condicionado por un campo de la configuracin
	 * @param pNode es el <code>Node</code> que contiene el nodo a tratar de una imagen (dibujo)
	 */
	private String procesaDraw(Node pNode) {
		StringBuilder sb = new StringBuilder();
		int numNodos;
		OdfElement elementoTrt = (OdfElement) pNode;
		
		numNodos = elementoTrt.getLength();
		// Si se trata de un elemento vacio, no tenemos texto
		if (numNodos == 0) {
			return "";
		}
		NodeList nodeList = elementoTrt.getChildNodes();
		int i;
		for (i = 0; i < nodeList.getLength(); i++) {
			Node node;
			node = nodeList.item(i);
			if (node.getNodeType() == Node.TEXT_NODE)
				sb.append(node.getNodeValue());
			else if (node.getNodeType() == Node.ELEMENT_NODE) {
				if (node instanceof TextHElement || node instanceof TextPElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(' ');
						sb.append(obtenerTexto((OdfElement)node));
					}
				}
				else if (node instanceof TextListElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(' ');
						sb.append(obtenerTexto((OdfElement)node));
					}
				}
				else if (node instanceof TableTableElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoTabla((OdfElement)node));
					}
				}
				else if (node instanceof DrawTextBoxElement) {
						// Se evita tratar elementos vacios
						if (((OdfElement)node).getLength() > 0) {
							sb.append(' ');
							sb.append(procesaDraw(node));
						}
				}
				else if (!(node instanceof OfficeAnnotationElement)) {
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(procesaDraw(node));		
					}
				}
			}
		}
		return sb.toString();
	}
	
	
	/**
	 * Proceso para buscar el texto a convertir en voz, a partir de un elemento
	 * buscando el contenido de sus hijos
	 * @param odfElement es el <code>OdfElement</code> que contiene el nodo a tratar
	 * @return <code>String</code> con el texto obtenido
	 */
	private String obtenerTexto(OdfElement odfElement) {
		StringBuilder sb = new StringBuilder();
		
		NodeList nodeList = odfElement.getChildNodes();
		int i;
		for (i = 0; i < nodeList.getLength(); i++) {
			Node node;
			node = nodeList.item(i);
			if (node.getNodeType() == Node.TEXT_NODE)
				sb.append(node.getNodeValue());
			else if (node.getNodeType() == Node.ELEMENT_NODE) {
				if (node instanceof TextSpanElement) {
					sb.append(((TextSpanElement) node).getTextContent());
				}
				else if (node.getNodeName().equals("text:s")) {
					Integer count = ((TextSElement) node).getTextCAttribute();
					for (int j = 0; j < (count != null ? count : 0); j++)
						sb.append(' ');
				} else if (node.getNodeName().equals("text:tab"))
					sb.append('\t');
				else if (node.getNodeName().equals("text:line-break")) {
					String lineseperator = System.getProperty("line.separator");
					sb.append(lineseperator);
				} else if (node.getNodeName().equals("text:a"))
					sb.append(TextHyperlink.getInstanceof((TextAElement) node).getTextContent());
				else if (node instanceof DrawFrameElement || node instanceof Dr3dSceneElement) {
						if (analiser.getImagen()) {
							sb.append(procesaDraw(node));
						}
				}
				else if (node instanceof DrawCustomShapeElement || node instanceof DrawAElement) {
						if (analiser.getImagen()) {
							sb.append(procesaDraw(node));
						}
				}
				else if (node instanceof DrawCaptionElement || node instanceof DrawCircleElement) {
						if (analiser.getImagen()) {
							sb.append(procesaDraw(node));
						}
				}
				else if (node instanceof TextNoteElement) {
					if (analiser.getNota()) {
						TextNoteElement textNote = (TextNoteElement)node;
						TextNoteBodyElement textbody = TextNoteElement.findFirstChildNode(TextNoteBodyElement.class, textNote);
						sb.append(' ');
						sb.append(textbody.getTextContent());
						sb.append(' ');
					}
				}
				else if (node instanceof DrawConnectorElement || node instanceof DrawControlElement ||
						 node instanceof DrawCustomShapeElement || node instanceof DrawEllipseElement ||
						 node instanceof DrawGElement || node instanceof  DrawLineElement ||
						 node instanceof DrawMeasureElement || node instanceof  DrawPageThumbnailElement ||
						 node instanceof DrawPathElement || node instanceof  DrawPolygonElement ||
						 node instanceof DrawPolylineElement || node instanceof DrawRectElement ||
						 node instanceof DrawRegularPolygonElement) {
					if (analiser.getImagen()) {
						sb.append(procesaDraw(node));
					}
				}
				else if (!(node instanceof OfficeAnnotationElement)) {
					if (node instanceof TextListHeaderElement) { 
						// Se deja un espacio para separar textos de las listas, sino pronuncia todo junto
						sb.append(' ');
					}
					if (node instanceof TextListItemElement) {
						// Se deja un espacio para separar textos de las listas, sino pronuncia todo junto
						sb.append('\n');
					}
					if (((OdfElement)node).getLength() > 0) { // Se evita tratar elementos vacios
						sb.append(obtenerTexto((OdfElement)node));
					}
				}
			}
		}
		return sb.toString();
	}
	
	/**
	 * Proceso para buscar el texto en una tabla para convertir en voz, a partir de un elemento
	 * buscando el contenido de sus hijos
	 * @param odfElement es el <code>OdfElement</code> que contiene el nodo a tratar
	 * @return <code>String</code> con el texto obtenido
	 */
	private String obtenerTextoTabla(OdfElement odfElement) {
		StringBuilder sb = new StringBuilder();
		
		NodeList nodeList = odfElement.getChildNodes();
		int i;
		for (i = 0; i < nodeList.getLength(); i++) {
			Node node;
			node = nodeList.item(i);
			if (node.getNodeType() == Node.TEXT_NODE)
				sb.append(node.getNodeValue());
			else if (node.getNodeType() == Node.ELEMENT_NODE) {
				if (node instanceof TableTableRowElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
					sb.append(obtenerTextoTabla((OdfElement)node));
					}
				}
				if (node instanceof TableTableCellElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						// Seguro que podemos tener texto, siempre que no se trate de una celda vacia
						sb.append('\n');
						sb.append(obtenerTexto((OdfElement)node));
					}
				}
				if (node instanceof TableTableRowsElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoTabla((OdfElement)node));
					}
				}
				if (node instanceof TableDescElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append('\n');
						sb.append(obtenerTexto((OdfElement)node));
					}
				}
				if (node instanceof TableTableHeaderRowsElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoTabla((OdfElement)node));
					}
				}
				if (node instanceof TableCoveredTableCellElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append('\n');
						sb.append(obtenerTexto((OdfElement)node));
					}
				}
				if (node instanceof TableTableRowGroupElement) { 
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoTabla((OdfElement)node));
					}
				}

			}
		}
		return sb.toString();
	}

	/**
	 * Proceso para buscar el texto en una tabla de contenido (ndices) para convertir en voz, a partir de un elemento
	 * buscando el contenido de sus hijos
	 * @param odfElement es el <code>OdfElement</code> que contiene el nodo a tratar
	 * @return <code>String</code> con el texto obtenido
	 */
	private String obtenerTextoIndice(OdfElement odfElement) {
		StringBuilder sb = new StringBuilder();
		
		NodeList nodeList = odfElement.getChildNodes();
		int i;
		for (i = 0; i < nodeList.getLength(); i++) {
			Node node;
			node = nodeList.item(i);
			if (node.getNodeType() == Node.TEXT_NODE)
				sb.append(node.getNodeValue());
			else if (node.getNodeType() == Node.ELEMENT_NODE) {
				if (node instanceof TextIndexBodyElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoIndice((OdfElement)node));
					}
				}
				if (node instanceof TextAlphabeticalIndexElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoIndice((OdfElement)node));
					}
				}
				if (node instanceof TextBibliographyElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoIndice((OdfElement)node));
					}
				}
				if (node instanceof TextHElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(' ');
						sb.append(obtenerTexto((OdfElement)node));
					}
				}
				if (node instanceof TextPElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(' ');
						sb.append(obtenerTexto((OdfElement)node));
					}
				}
				if (node instanceof TextIllustrationIndexElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoIndice((OdfElement)node));
					}
				}
				if (node instanceof TextIndexTitleElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(' ');
						sb.append(obtenerTexto((OdfElement)node));
					}
				}
				if (node instanceof TextListElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(' ');
						sb.append(obtenerTexto((OdfElement)node));
					}
				}
				if (node instanceof TextNumberedParagraphElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoIndice((OdfElement)node));
					}
				}
				if (node instanceof TextNumberElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(' ');
						sb.append(obtenerTexto((OdfElement)node));
					}
				}
				if (node instanceof TextObjectIndexElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoIndice((OdfElement)node));
					}
				}
				if (node instanceof TableTableElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(' ');
						sb.append(obtenerTextoTabla((OdfElement)node));
					}
				}
				if (node instanceof TextTableIndexElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoIndice((OdfElement)node));
					}
				}
				if (node instanceof TextTableOfContentElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoIndice((OdfElement)node));
					}
				}
				if (node instanceof TextUserIndexElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoIndice((OdfElement)node));
					}
				}
				if (node instanceof TextSectionElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoIndice((OdfElement)node));
					}
				}
			}
		}
		return sb.toString();
	}

	/**
	 * Proceso para buscar el texto en una seccin para convertir en voz, a partir de un elemento
	 * buscando el contenido de sus hijos
	 * @param odfElement es el <code>OdfElement</code> que contiene el nodo a tratar
	 * @return <code>String</code> con el texto obtenido
	 */
	private String obtenerTextoSeccion(OdfElement odfElement) {
		StringBuilder sb = new StringBuilder();
		
		NodeList nodeList = odfElement.getChildNodes();
		int i;
		for (i = 0; i < nodeList.getLength(); i++) {
			Node node;
			node = nodeList.item(i);
			if (node.getNodeType() == Node.TEXT_NODE)
				sb.append(node.getNodeValue());
			else if (node.getNodeType() == Node.ELEMENT_NODE) {
				if (node instanceof TableTableElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(' ');
						sb.append(obtenerTextoTabla((OdfElement)node));
					}
				}
				if (node instanceof TextAlphabeticalIndexElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoIndice((OdfElement)node));
					}
				}
				if (node instanceof TextBibliographyElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoSeccion((OdfElement)node));
					}
				}
				if (node instanceof TextIndexBodyElement || node instanceof TextTableOfContentElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoIndice((OdfElement)node));
					}
				}
				if (node instanceof TextHElement || node instanceof TextPElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(' ');
						sb.append(obtenerTexto((OdfElement)node));
					}
				}
				if (node instanceof TextIllustrationIndexElement || node instanceof TextUserIndexElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoIndice((OdfElement)node));
					}
				}
				if (node instanceof TextListElement || node instanceof TextNumberElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(' ');
						sb.append(obtenerTexto((OdfElement)node));
					}
				}
				if (node instanceof TextNumberedParagraphElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoSeccion((OdfElement)node));
					}
				}
				if (node instanceof TextObjectIndexElement || node instanceof TextTableIndexElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoIndice((OdfElement)node));
					}
				}
				if (node instanceof TextSectionElement) { 
					// Se evita tratar elementos vacios
					if (((OdfElement)node).getLength() > 0) {
						sb.append(obtenerTextoSeccion((OdfElement)node));
					}
				}
			}
		}
		return sb.toString();
	}
		
}
