package Compiladores;

import java.io.File;	
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLEncoder;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.swing.JOptionPane;

import us.monoid.web.BinaryResource;
import us.monoid.web.Resty;


/**
 * Clase para generar los archivos de audio a partir del texto que recibe
 * @author  Enrique Marquet Durn
 * @version 1.0
 */
public class ProcesoTTS {
	private String nomArch;        // Contendr el nombre del archivo que guardar el audio
	private Boolean nuevoArch;     // Si tiene valor true se tiene que crear, con false debe aadir la nueva informacin
	private Path dirTmp;           // Directorio de trabajo para guardar los audios temporales

	/**
	 * Constructor de la clase
	 * @param nomArch el <code>String</code> con el path y nombre donde hay que guardar el audio
	 * @param dirTmp es el <code>Path</code> con el directorio temporal para guardar los archivos intermedios de audio
	 */
	public ProcesoTTS (String nomArch, Path dirTmp) {
		this.nomArch = new String (nomArch);
		this.nuevoArch = true;
		this.dirTmp = dirTmp;
	}

	/**
	 * Codifica el texto para llamar al mtodo que crea el audio
	 * @param text es el <code>String</code> que contiene el texto para generar el audio
	 * @param language es el <code>String</code> que contiene el idioma de la voz que se utilizar
	 */
	public void textToSpeech(String text, String language){
		//Codifica el texto a cdigo URL con utf-8 y crea snippets de 100 caracteres 
		//o menos basandose en el texto codificado
		try {
			if (text.length() == 0) {
				return;
			}
			text = URLEncoder.encode(text, "utf-8");
		} catch (UnsupportedEncodingException e) {
			JOptionPane.showMessageDialog(null, e, "Error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		ArrayList<String> paramSnippets = getParams(text);
		//crea el fichero mp3 que tendr la versin audio del texto, despus se transforma en archivo AIFF 
		String destination = Paths.get(dirTmp.toString(), Paths.get(nomArch).toFile().getName()).toString();
		makeAudio(destination, language, paramSnippets);
	}

	/**
	 * Mtodo para transformar el audio obtenido de Google en MP3 a formato AIFF
	 */
	public void generaAif() {
		String destination = Paths.get(dirTmp.toString(), Paths.get(nomArch).toFile().getName()).toString();
		File file = new File(destination + ".mp3");
		// Si el fichero no existe no hay que hacer la transformacin a AIFF
		if (!file.exists())
			return;
		AudioFileFormat aff;
		
		try {
			aff = AudioSystem.getAudioFileFormat(file);
			AudioFormat baseFormat = aff.getFormat();
			AudioInputStream ini = AudioSystem.getAudioInputStream(file);
			AudioFormat decodedFormat =  new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, baseFormat.getSampleRate(), 16, 1, 2,
	                baseFormat.getSampleRate(),
	                true);
			AudioInputStream din = null;
			din = AudioSystem.getAudioInputStream(decodedFormat, ini);
			AudioSystem.write(din, AudioFileFormat.Type.AIFF, new File(nomArch + ".aif"));
			ini.close();
			din.close();
			if (Files.deleteIfExists(file.toPath()))
				System.out.println("Borrado correcto..........");
			else
				System.out.println("Borrado incorrecto..........");
		} catch (UnsupportedAudioFileException e) {
			JOptionPane.showMessageDialog(null, e);
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, e);
		}
		
	}

	/**
	 * Crea o aade mp3 a un fichero basandose en el contenido del texto e idioma.
	 * @param destiantion es el <code>String</code> con el nombre del path donde se guarda el audio mp3
	 * @param language es el <code>String</code> con el cdigo del idioma (en, es, zh, etc)
	 * @param snippets es la <code>List<String></code> lista de cadenas a traducir menores o iguales a 100 caracteres
	 */
	private void makeAudio(String destination, String language,
			List<String> snippets) {
		
		try {
			// Se debe aadir al fichero o se debe crear
			boolean shouldAppend = true;
			// Se comprueba primero el indicador de nuevo archivo
			// En el caso de que sea cierto, se deber crear el archivo, caso contrario se abre el fichero para aadir audio
			OutputStream os;
			if (nuevoArch) {
				os = new FileOutputStream(new File(destination+ ".mp3"));
				nuevoArch = false;
			} else {
				os = new FileOutputStream(new File(destination + ".mp3"),true);
				// dentro del bucle ya no habr que especificar que el FileOutputStream se aade al file
				shouldAppend = false;
			}
			File tempFile = File.createTempFile("FichAudio", ".mp3");
			tempFile.deleteOnExit();

			for (String snippet : snippets) {
				//Cadena con la URL que utilizaremps con el texto (snippet) y el idioma (language) 
				String urlString = new String ("http://www.translate.google.com/translate_tts?ie=UTF-8&q=" + snippet + "&tl=" + language+"&prev=input");
				BinaryResource res = new Resty().bytes(new URI(urlString));
				res.save(tempFile);
				
				// Copia fichero temporal al fichero que contendr todo el audio
				Files.copy(Paths.get(tempFile.getCanonicalPath()), os);
				//se asigna os a un FileOutputStream que aade audio al fichero de destino
				//slo se tiene que hacer una vez ya que despus se utiliza la misma referencia
				if (shouldAppend) {
					os.close(); //cierra la referencia al output streams
					os = new FileOutputStream(new File(destination + ".mp3"),true);
					shouldAppend = false;
				}
			}
			//cierra la referencia al output streams
			os.close();
		} catch (FileNotFoundException e) {
			JOptionPane.showMessageDialog(null, e);
			return;
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, e);
			return;
		} catch (URISyntaxException e) {
			JOptionPane.showMessageDialog(null, e);
		}
	}
	/** 
	 * Coge el texto de un String convertido en formato URL, y lo separa en un ArrayList de Strings cada uno 
	 * con una longitud mxima de 100 caracteres incluyendo los signos '+'.
	 * @param text Es el <code>String</code> que queremos converir a audio mediante Google's text to speech
	 * @return result que es un <code>ArrayList<String></code> con cadenas de dimensin igual o inferior a 100
	 */
	private static ArrayList<String> getParams(String text) {
		//El texto recibido debe ser repartido en cadenas de hasta 100 caracteres 
		String paramText = text;
		// Indice inicial de la actual cadena en relacin con la que se ha recibido
		int startIndex = 0;
		// variable que devolveremos que contendr las cadenas con longitud menor o igual a 100 caracteres
		ArrayList<String> result = new ArrayList<String>();
		// Se realiza el proceso hasta que el indice inicial (startIndex) no ha llegado al ltimo caracter del texto recibido
		while (startIndex < text.length() - 1) {
			//Obtiene el ndice final de la cadena, que es uno ms del indice del ltimo caracter que es <=100 
			//caracteres alejados del inicio de la cadena que no es '+'
			// (indice + 1 porque el parmetro del ndice final String.substring(int,int) no es inclusive
			int endIndex = getSnippetEnd(paramText, startIndex);
			// Se obtine la subcadena (snippet) a partir del String recibido usando los indices inicial y final encontrados
			String snippet = paramText.substring(startIndex, endIndex);
			//ensure that the snippet of text has at least 2 characters 
			//(sometimes there may be a '+' at the beginning of the substring)
			// Nos aseguramos que la cadena de texto tiene por lo menos 2 caracteres
			// para sacar el posible signo '+' que pueda haber al inicio de la cadena
			if (snippet.length() >= 2) {
				if (snippet.substring(0, 1).equals("+")) {
					snippet = snippet.substring(1);
				}
			}
			// Se aade la nueva subcadena al array de cadenas del resultado
			result.add(snippet);
			// ponemos como ndice inicial el valor del actual valor del ndice final
			startIndex = endIndex;
		}
		// una vez tenemos todo el texto inicial troceado, devolvemos el resultado
		return result;
	}

	/**
	 * 
	 * @param text es el <code>String</code> que estamos converitiendo a audio
	 * @param startIndex es <code>int</code> el ndice inicial de texto que vamos a coger de la cadena text
	 * @return devuelve la posicin final del texto que se tendr que seleccionar
	 */
	private static int getSnippetEnd(String text, int startIndex) {
		// Asignamos a subtext el texto candidato para la siguiente cadena
		// Si nos queda por lo menos 100 caracteres en el texto original desde el ndice (startIndex), asignamos
		// a subtext 100 caracteres, en el caso contrario le asignamos los caracteres que queden hasta el final
		String subtext = (text.substring(startIndex).length() > 100) ? 
		                   text.substring(startIndex, startIndex + 100) : 
		                   text.substring(startIndex);
		                   
		// la variable end contiene inicialmente el final del ndice como la longitud de la cadena de texto 
		// nos dar el final del texto que tendremos que seleccionar del texto original en el mtodo getParams(String)
		int end = subtext.length();
		// Si el texto no llega a 100 caracteres, ya tenemos el resultado.
		// En caso contrario, nos tenemos que cerciorar que no estemos en el medio de una palabra.
		// Si lo estamos, debemos decrementar el ndice 'end' hasta encontrar un signo '+'.
		// Simplemente eliminamos el '+', y no tendremos problemas de cortar una palabra en la conversin
		if (text.substring(startIndex).length() > 100) {
				while (!subtext.substring(subtext.length() - 1, subtext.length()).equals("+")) {
					subtext = subtext.substring(0, subtext.length() - 1);
					end--;
				}
				// el signo '+' no lo tenemos en cuenta
				end--;
		}
		// Devuelve el ndice final de la subcadena ms el de partida de la cadena original
		// esto nos da el ndice final del texto original
		return end + startIndex;
	}

	/**
	 * Getter para obtener el nombre del archivo de audio que se est generando
	 * @return the nomArch
	 */
	public String getNomArch() {
		return nomArch;
	}

	/**
	 * Setter para el nombre actual del archivo de audio que se tiene que generar
	 * @param nomArch el <String> nomArch es el nuevo archivo de audio a generar
	 */
	public void setNomArch(String nomArch) {
		this.nomArch = nomArch;
	}

	/**
	 * Getter del valor <Boolean> por si hay que crear el archivo nuevo o aadir a uno que ya existe
	 * @return the nuevoArch
	 */
	public Boolean getNuevoArch() {
		return nuevoArch;
	}

	/**
	 * Setter del valor <Boolean> por si hay que crear el archivo nuevo o aadir a uno que ya existe
	 * @param nuevoArch the nuevoArch to set
	 */
	public void setNuevoArch(Boolean nuevoArch) {
		this.nuevoArch = nuevoArch;
	}
}
